# mcctl
Mission Control CLI components

## Installation

Install first the migration CRD:

```kubectl apply -f config/crd/datastax.com_clusterconfigs```

Then proceed with the migration workflow:

```console
mcctl import init
mcctl import add
mcctl import commit
```

## Development

To build mcctl, simply run ``make``. It will generate the manifests (if necessary), vet, fmt, lint and test. Tested to work on amd64 and aarch64 architectures (Linux). 

## Testing

This repository has both envtests as well as unit tests that don't require envtest. When a package requires envtesting capabilities, add the following (once per package):

 ```go
 import "github.com/riptano/mcctl/internal/envtest"

 var (
	env *envtest.Environment
)

func TestMain(m *testing.M) {
	os.Exit(envtest.Run(m, func(e *envtest.Environment) { env = e }))
}
 ```

 ``testfiles`` directory has testing related files, such as configurations to be parsed. Add more as necessary, but keep in mind that some tests assume there are certain files. Use sub directories if it makes sense.

 If a test-Kubernetes is required, there's a simple k3d configuration file in the ``.github/helpers``, that creates a local Kubernetes cluster with hostNetworking and hostPath for the DSE package installations.

## Publishing releases

In order to publish releases, tag the commit with the version number, and  push the tag. Go to the [tags page on GitHub](https://github.com/adejanovski/mcctl/tags) and create a release from the tag. Wait for the action to go through and attach the built binary to the release in tar.gz format.
Download the new binary from the release page, and upload them through the [Manual Upload to Repository](https://jenkins-release.aws.dsinternal.org/job/manually_upload_to_repo/) Jenkins job. This requires to be connected to the AWS Lab VPN and to have the "Jenkins (Cloudbees EKS)" chicklet in your Okta account.  
Upload the binary to the `public` repository and use `tar/mission-control` as value for `Path_To_File`.  
Give some time for the job to complete, and then the new version will be available under: `http://repo-public.aws.dsinternal.org/tar/mission-control/<name of the binary>`

## Structure and k8ssandra-client

If there are tools that are generic and not strictly related to mcctl's non-OSS features, write them to k8ssandra-client. If they have something to do with Replicated, migrations, mission-control then write them here.

### APIs

The apis/ directory is for resources that are external, such as the ``ClusterConfig`` CRD which is used to store the status of migrations in the Kubernetes. Internal types are not meant to be stored here, keep them in the ``pkg``.

### Command line

Command line interface is defined in the ``cmd/``, while implementations should be in ``pkg``. Try to avoid polluting ``pkg`` with command line stuff, since that helps us to refactor the command line interface later if necessary. There are places currently in pkg which print stuff for command line output since that's how they were in the PoC, but this is to be refactored at some point. The command line interface uses Cobra for command line parsing.

### internal/

This is used for testing packages to avoid circular dependencies.

## Migration code

Migration code is located in ``pkg/migrate``, with few sub-directories. ``config`` includes all the configuration management, while ``storage`` has all the PVC / PV / mounting stuff.

Inside  ``config``, there's also ``config/generated`` and ``config/types`` which are built using the [definitions-parser](https://github.com/burmanm/definitions-parser). That one requires to check-out [cass-config-definitions](https://github.com/datastax/cass-config-definitions). Also, ``matcher.go`` is from ``definitions-parser``. If you make changes to these, please push to upstream and re-generate from there - don't modify by hand directly here.

Otherwise, to understand the migration code, read from the ``cmd/mcctl/import`` and follow the call paths from there.